package handler

import (
	"gitlab.informatika.org/k-01-11/graduit-be/src/database"
	batchSeminarConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_seminar/config"
	batchSeminarTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_seminar/transport"
	batchSidangConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_sidang/config"
	batchSidangTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_sidang/transport"
	bimbinganLogConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/config"
	bimbinganLogTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/transport"
	dashboardDosbingConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_dosbing/config"
	dashboardDosbingTrasport "gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_dosbing/transport"
	jadwalSeminarConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_seminar/config"
	jadwalSeminarTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_seminar/transport"
	jadwalSidangConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_sidang/config"
	jadwalSidangTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_sidang/transport"
	pendaftaranConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/config"
	pendaftaranTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/transport"
	statusMahasiswaConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/status_mahasiswa/config"
	statusMahasiswaTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/status_mahasiswa/transport"
	systemLogConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/system_log/config"
	systemLogTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/system_log/transport"
	topicAllocationConfig "gitlab.informatika.org/k-01-11/graduit-be/src/module/topic_allocation/config"
	topicAllocationTransport "gitlab.informatika.org/k-01-11/graduit-be/src/module/topic_allocation/transport"
)

type Service struct {
	InternalSystemLogHandler     *systemLogTransport.InternalSystemLogHandler
	AdminSytemLogHandler         *systemLogTransport.AdminSystemLogHandler
	AdminBimbinganLogHandler     *bimbinganLogTransport.AdminBimbinganLogHandler
	AdminTopicAllocationHandler  *topicAllocationTransport.AdminTopicAllocationHandler
	BatchSeminarHandler          *batchSeminarTransport.AdminBatchSeminarHandler
	BatchSidangHandler           *batchSidangTransport.AdminBatchSidangHandler
	JadwalSidangHandler          *jadwalSidangTransport.AdminJadwalSidangHandler
	JadwalSeminarHandler         *jadwalSeminarTransport.AdminJadwalSeminarHandler
	AdminStatusMahasiswaHandler  *statusMahasiswaTransport.AdminStatusMahasiswaHandler
	AdminDashboardDosbingHandler *dashboardDosbingTrasport.AdminDashboardDosbingHandler
	AdminPendaftaranHandler     *pendaftaranTransport.AdminPendaftaranHandler
}

func MakeHandler() *Service {
	dbWrite := database.GetInstance()
	dbRead := database.GetInstance()

	InternalSystemLogHandler := systemLogTransport.NewInternalSystemLogHandler(systemLogConfig.SystemLogTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminSystemLogHandler := systemLogTransport.NewAdminSystemLogHandler(systemLogConfig.SystemLogTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminBimbinganLogHandler := bimbinganLogTransport.NewAdminBimginganLogHandler(bimbinganLogConfig.BimbinganLogTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminTopicAllocationHandler := topicAllocationTransport.NewAdminTopicAllocationHandler(topicAllocationConfig.TopicAllocationTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminBatchSeminarHandler := batchSeminarTransport.NewAdminBatchSeminarHandler(batchSeminarConfig.BatchSeminarTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminBatchSidangHandler := batchSidangTransport.NewAdminBatchSidangHandler(batchSidangConfig.BatchSidangTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminJadwalSidangHandler := jadwalSidangTransport.NewAdminJadwalSidangHandler(jadwalSidangConfig.JadwalSidangTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminJadwalSeminarHandler := jadwalSeminarTransport.NewAdminJadwalSeminarHandler(jadwalSeminarConfig.JadwalSeminarTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminStatusMahasiswaHandler := statusMahasiswaTransport.NewAdminStatusMahasiswaHandler(statusMahasiswaConfig.StatusMahasiswaTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminDashboardDosbingHandler := dashboardDosbingTrasport.NewAdminDashboardDosbingHandler(dashboardDosbingConfig.DashboardDosbingTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})
	AdminPendaftaranHandler := pendaftaranTransport.NewAdminPendaftaranHandler(pendaftaranConfig.PendaftaranTransportConfig{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	})

	return &Service{
		InternalSystemLogHandler:     InternalSystemLogHandler,
		AdminSytemLogHandler:         AdminSystemLogHandler,
		AdminBimbinganLogHandler:     AdminBimbinganLogHandler,
		AdminTopicAllocationHandler:  AdminTopicAllocationHandler,
		BatchSeminarHandler:          AdminBatchSeminarHandler,
		BatchSidangHandler:           AdminBatchSidangHandler,
		JadwalSidangHandler:          AdminJadwalSidangHandler,
		JadwalSeminarHandler:         AdminJadwalSeminarHandler,
		AdminStatusMahasiswaHandler:  AdminStatusMahasiswaHandler,
		AdminDashboardDosbingHandler: AdminDashboardDosbingHandler,
		AdminPendaftaranHandler:     AdminPendaftaranHandler,
	}
}
