package transport

import (
	"fmt"
	"net/http"

	"github.com/labstack/echo"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_seminar/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_seminar/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_seminar/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/batch_seminar/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminBatchSeminarHandler struct {
	batchSeminarUsecase usecase.BatchSeminarUsecase
}

func NewAdminBatchSeminarHandler(cfg config.BatchSeminarTransportConfig) *AdminBatchSeminarHandler {
	batchSeminarRepository := repository.NewBatchSeminarRepository(cfg.DBWrite, cfg.DBRead)
	batchSeminarUsecase := usecase.NewBatchSeminarUseCase(batchSeminarRepository)

	return &AdminBatchSeminarHandler{
		batchSeminarUsecase: batchSeminarUsecase,
	}

}

func (t *AdminBatchSeminarHandler) MountAdmin(group *echo.Group) {
	group.GET("/batch-seminar", t.AddBatchSeminar)
	group.GET("/batch-seminar-get", t.GetBatchSeminar)
	group.GET("/batch-seminar-delete", t.DeleteBatchSeminar)

}

func (t *AdminBatchSeminarHandler) AddBatchSeminar(c echo.Context) error {
	batch := new(entity.BatchSeminar)

	start := c.QueryParam("startDate")
	end := c.QueryParam("endDate")
	fmt.Println(start)
	fmt.Println(end)

	target := &entity.BatchSeminar{
		StartDate: start,
		EndDate:   end,
	}

	if err := c.Bind(batch); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	fmt.Println(*batch)
	newBatch, err := t.batchSeminarUsecase.AddBatchSeminar(*target)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Batch added successfully", newBatch))
}

func (t *AdminBatchSeminarHandler) GetBatchSeminar(c echo.Context) error {
	listGroupLimit, err := t.batchSeminarUsecase.GetBatchSeminar()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Batch seminar successfully retreived", listGroupLimit))
}
func (t *AdminBatchSeminarHandler) DeleteBatchSeminar(c echo.Context) error {
	id := c.QueryParam("id")
	listGroupLimit, err := t.batchSeminarUsecase.DeleteBatchSeminar(id)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Batch Seminar successfully deleted", listGroupLimit))
}
