package repository

import (
	"errors"

	"gitlab.informatika.org/k-01-11/graduit-be/src/models"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_dosbing/entity"
	"gorm.io/gorm"
)

type DashboardDosbingRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewDashboardDosbingRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *DashboardDosbingRepo {
	return &DashboardDosbingRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *DashboardDosbingRepo) GetMahasiswaByDosenId(idDosen string, offset, size int) (output []entity.MahasiswaBimbingan, err error) {
	var mahasiswaBimbingan []entity.MahasiswaBimbingan

	var dosen models.Dosen
	var mahasiswa []models.Mahasiswa
	var topik models.Topik
	var pendaftaran models.Pendaftaran

	result := repo.DBRead.Where("id_dosen = ?", idDosen).First(&dosen)
	if result.Error != nil {
		return nil, errors.New("dosen tidak ditemukan")
	}

	result = repo.DBRead.Limit(size).Offset(offset).Where("id_dosen = ?", dosen.IdDosen).Find(&mahasiswa)
	if result.Error != nil {
		return nil, result.Error
	}

	for _, mhs := range mahasiswa {

		repo.DBRead.Where("status = TRUE").First(&pendaftaran)
		repo.DBRead.Where("id = ?", pendaftaran.IdTopik).First(&topik)

		bimbingan := entity.MahasiswaBimbingan{
			IdMahasiswa: mhs.IdMahasiswa,
			Nama:        mhs.Name,
			NIM:         mhs.NIM,
			Topik:       topik.Judul,
			// TODO : KONEKIN KE STATUS
			Status: "LANCAR",
		}

		mahasiswaBimbingan = append(mahasiswaBimbingan, bimbingan)
	}

	return mahasiswaBimbingan, nil

}

func (repo *DashboardDosbingRepo) GetAmountMahasiswaByDosenId(idDosen string) (output []entity.JumlahMahasiswa, err error) {
	var jumlahMahasiswa entity.JumlahMahasiswa

	var dosen models.Dosen
	var mahasiswa []models.Mahasiswa

	result := repo.DBRead.Where("id_dosen = ?", idDosen).First(&dosen)
	if result.Error != nil {
		return nil, errors.New("dosen tidak ditemukan")
	}

	// Mahasiswa IF
	result = repo.DBRead.Where("id_dosen = ? AND NIM LIKE ?", dosen.IdDosen, "135%").Find(&mahasiswa)
	if result.Error != nil {
		jumlahMahasiswa.MahasiswaIF = 0
	}
	jumlahMahasiswa.MahasiswaIF = int(result.RowsAffected)

	// Mahasiswa STI
	result = repo.DBRead.Where("id_dosen = ? AND NIM LIKE ?", dosen.IdDosen, "182%").Find(&mahasiswa)
	if result.Error != nil {
		jumlahMahasiswa.MahasiswaSTI = 0
	}
	jumlahMahasiswa.MahasiswaSTI = int(result.RowsAffected)

	return []entity.JumlahMahasiswa{jumlahMahasiswa}, nil

}

func (repo *DashboardDosbingRepo) GetAmountStatusBimbinganMahasiswaByDosenId(idDosen string) (output []entity.StatusBimbinganMahasiswa, err error) {
	return nil, errors.New("Not implemented")
}
