package transport

import (
	"net/http"
	"strconv"

	"github.com/labstack/echo"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_dosbing/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_dosbing/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/dashboard_dosbing/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminDashboardDosbingHandler struct {
	dashboardDosbingUseCase usecase.DashboardDosbingUseCase
}

func NewAdminDashboardDosbingHandler(cfg config.DashboardDosbingTransportConfig) *AdminDashboardDosbingHandler {
	dashboardDosbingRepository := repository.NewDashboardDosbingRepository(cfg.DBWrite, cfg.DBRead)
	dashboardDosbingUsecase := usecase.NewDashboardDosbingUsecase(dashboardDosbingRepository)

	return &AdminDashboardDosbingHandler{
		dashboardDosbingUseCase: dashboardDosbingUsecase,
	}
}

func (t *AdminDashboardDosbingHandler) MountAdmin(group *echo.Group) {
	group.GET("/dashboard-dosbing", t.GetMahasiswaByDosenId)
	group.GET("/dashboard-dosbing/jurusan", t.GetAmountMahasiswaByDosenId)
	group.GET("/dashboard-dosbing/bimbingan", t.GetAmountStatusBimbinganMahasiswaByDosenId)
}

func (t *AdminDashboardDosbingHandler) GetMahasiswaByDosenId(c echo.Context) error {
	idDosen := c.QueryParam("id_dosen")
	sizeStr := c.QueryParam("limit")
	offsetStr := c.QueryParam("offset")

	if idDosen == "" {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "insert id_dosen", nil))
	}

	size, err := strconv.Atoi(sizeStr)
	if err != nil {
		// Handle error, maybe return an error response
		size = 10
	}

	offset, err := strconv.Atoi(offsetStr)
	if err != nil {
		// Handle error, maybe return an error response
		offset = 0
	}

	listMahasiswa, err := t.dashboardDosbingUseCase.GetMahasiswaByDosenId(idDosen, offset, size)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "List mahasiswa successfully retreived", listMahasiswa))

}

func (t *AdminDashboardDosbingHandler) GetAmountMahasiswaByDosenId(c echo.Context) error {
	idDosen := c.QueryParam("id_dosen")

	if idDosen == "" {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "insert id_dosen", nil))
	}

	listMahasiswa, err := t.dashboardDosbingUseCase.GetAmountMahasiswaByDosenId(idDosen)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "List mahasiswa successfully retreived", listMahasiswa[0]))

}

func (t *AdminDashboardDosbingHandler) GetAmountStatusBimbinganMahasiswaByDosenId(c echo.Context) error {
	idDosen := c.QueryParam("id_dosen")

	if idDosen == "" {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "insert id_dosen", nil))
	}

	listMahasiswa, err := t.dashboardDosbingUseCase.GetAmountStatusBimbinganMahasiswaByDosenId(idDosen)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "List mahasiswa successfully retreived", listMahasiswa[0]))

}
