package repository

import (
	// "time"

	"github.com/google/uuid"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_seminar/entity"
	"gorm.io/gorm"
)

type JadwalSeminarRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewJadwalSeminarRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *JadwalSeminarRepo {
	return &JadwalSeminarRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *JadwalSeminarRepo) NewJadwalSeminar(jadwal entity.JadwalSeminar) (output entity.JadwalSeminar, err error) {
	jadwal.ID = uuid.New().String()

	result := repo.DBWrite.Create(&jadwal)
	if result.Error != nil {
		return entity.JadwalSeminar{}, result.Error
	}
	return jadwal, nil
}

func (repo *JadwalSeminarRepo) GetJadwalSeminarByBatch(idBatch string) (output []entity.JadwalSeminar, err error) {
	var jadwalSeminar []entity.JadwalSeminar
	result := repo.DBRead.Where("id_batch = ?", idBatch).Find(&jadwalSeminar)
	if result.Error != nil {
		return nil, err
	}
	return jadwalSeminar, nil
}

func (repo *JadwalSeminarRepo) GetLatestJadwalSeminar() (output []entity.JadwalSeminar, err error) {
	var jadwalSeminar []entity.JadwalSeminar
	err = repo.DBRead.
		Where("id_batch = (SELECT MAX(time) FROM jadwal_seminars)").
		Find(&jadwalSeminar).Error
	if err != nil {
		return nil, err
	}
	return jadwalSeminar, nil
}

func (repo *JadwalSeminarRepo) UpdateJadwalSeminar(jadwalSeminar entity.JadwalSeminar) (entity.JadwalSeminar, error) {
	err := repo.DBWrite.Save(&jadwalSeminar).Error
	if err != nil {
		return jadwalSeminar, err
	}
	return jadwalSeminar, nil
}

func (repo *JadwalSeminarRepo) DeleteJadwalSeminar(id string) (entity.JadwalSeminar, error) {
	jadwalSeminar := entity.JadwalSeminar{ID: id}
	err := repo.DBWrite.Delete(&jadwalSeminar).Error

	if err != nil {
		return jadwalSeminar, err
	}

	return jadwalSeminar, err
}

func (repo *JadwalSeminarRepo) GetJadwalSeminarAll() (output []entity.JadwalSeminar, err error) {
	var jadwalSeminar []entity.JadwalSeminar
	result := repo.DBRead.Find(&jadwalSeminar)
	if result.Error != nil {
		return nil, err
	}
	return jadwalSeminar, nil
}