package transport

import (
	"fmt"
	"net/http"

	"github.com/labstack/echo"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_seminar/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_seminar/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_seminar/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_seminar/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
	"gopkg.in/gomail.v2"
)

type AdminJadwalSeminarHandler struct {
	jadwalSeminarUsecase usecase.JadwalSeminarUsecase
}

func NewAdminJadwalSeminarHandler(cfg config.JadwalSeminarTransportConfig) *AdminJadwalSeminarHandler {
	jadwalSeminarRepository := repository.NewJadwalSeminarRepository(cfg.DBWrite, cfg.DBRead)
	jadwalSeminarUsecase := usecase.NewJadwalSeminarUseCase(jadwalSeminarRepository)

	return &AdminJadwalSeminarHandler{
		jadwalSeminarUsecase: jadwalSeminarUsecase,
	}

}

func (t *AdminJadwalSeminarHandler) MountAdmin(group *echo.Group) {
	group.POST("/jadwal-seminar", t.AddJadwalSeminar)
	group.GET("/jadwal-seminar-get", t.GetJadwalSeminarByBatch)
	group.GET("/jadwal-seminar-latest", t.GetLatestJadwalSeminar)
	group.GET("/jadwal-seminar-delete", t.DeleteJadwalSeminar)
	group.POST("/jadwal-seminar-edit", t.EditJadwalSeminar)
	group.GET("/jadwal-seminar-all", t.GetJadwalSeminarAll)
}

func (t *AdminJadwalSeminarHandler) AddJadwalSeminar(c echo.Context) error {
	request := new(entity.JadwalSeminarRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	targetJadwalSeminar := &entity.JadwalSeminar{
		Name:        request.Name,
		Nim:         request.Nim,
		NamaDosbing: request.NamaDosbing,
		Time:        request.Time,
		NamaDosuji1: request.NamaDosuji1,
		Ruang:       request.Ruang,
		Judul:       request.Judul,
		IDBatch:     request.IDBatch,
	}

	fmt.Println(targetJadwalSeminar)

	newJadwalSeminar, err := t.jadwalSeminarUsecase.AddJadwalSeminar(*targetJadwalSeminar)

	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	subject := "Penambahan Jadwal Seminar pada GraduIT"
	// headers := "MIME-version: 1.0;\nContent-Type: text; charset=\"UTF-8\";"
	bodyText := "Dengan surel ini, diberitahukan bahwa telah dilakukan penambahan jadwal seminar pada aplikasi GraduIT dengan informasi:\nNama:" + request.Name + "\nNIM: " + request.Nim + "\nNama Dosbing:" + request.NamaDosbing
	smtpHost := "smtp.gmail.com"
	smtpUsername := "graduititb@gmail.com"
	smtpPassword := "noim hyum imnm rgxw"

	m := gomail.NewMessage()
	m.SetHeader("From", "graduititb@gmail.com")
	m.SetHeader("To", "maximun154@gmail.com")
	m.SetHeader("Subject", subject)
	m.SetBody("text", bodyText)

	d := gomail.NewDialer(smtpHost, 587, smtpUsername, smtpPassword)

	err2 := d.DialAndSend(m)
	if err2 != nil {
		fmt.Println(err2)
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Jadwal added successfully", newJadwalSeminar))
}

func (t *AdminJadwalSeminarHandler) GetJadwalSeminarByBatch(c echo.Context) error {
	idBatch := c.QueryParam("id_batch")
	listGroupLimit, err := t.jadwalSeminarUsecase.GetJadwalSeminarByBatch(idBatch)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Seminar successfully retreived", listGroupLimit))
}

func (t *AdminJadwalSeminarHandler) GetLatestJadwalSeminar(c echo.Context) error {
	listGroupLimit, err := t.jadwalSeminarUsecase.GetLatestJadwalSeminar()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Latest Jadwal Seminar successfully retreived", listGroupLimit))
}

func (t *AdminJadwalSeminarHandler) DeleteJadwalSeminar(c echo.Context) error {
	idJadwal := c.QueryParam("id")
	listGroupLimit, err := t.jadwalSeminarUsecase.DeleteJadwalSeminar(idJadwal)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Seminar successfully deleted", listGroupLimit))
}

func (t *AdminJadwalSeminarHandler) EditJadwalSeminar(c echo.Context) error {
	request := new(entity.EditJadwalSeminarRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	fmt.Println(request)

	targetJadwalSeminar := &entity.JadwalSeminar{
		ID:          request.ID,
		Name:        request.Name,
		Nim:         request.Nim,
		NamaDosbing: request.NamaDosbing,
		Time:        request.Time,
		NamaDosuji1: request.NamaDosuji1,
		Ruang:       request.Ruang,
		Judul:       request.Judul,
		IDBatch:     request.IDBatch,
	}

	// Call the update method from the use case passing the targetJadwalSeminar
	updatedJadwalSeminar, err := t.jadwalSeminarUsecase.EditJadwalSeminar(*targetJadwalSeminar)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Seminar successfully edited", updatedJadwalSeminar))
}

func (t *AdminJadwalSeminarHandler) GetJadwalSeminarAll(c echo.Context) error {
	listGroupLimit, err := t.jadwalSeminarUsecase.GetJadwalSeminarAll()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Jadwal Seminar successfully retreived", listGroupLimit))
}
