package repository

import (
	// "time"

	"github.com/google/uuid"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/jadwal_sidang/entity"
	"gorm.io/gorm"
)

type JadwalSidangRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewJadwalSidangRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *JadwalSidangRepo {
	return &JadwalSidangRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *JadwalSidangRepo) NewJadwalSidang(jadwal entity.JadwalSidang) (output entity.JadwalSidang, err error) {
	jadwal.ID = uuid.New().String()

	result := repo.DBWrite.Create(&jadwal)
	if result.Error != nil {
		return entity.JadwalSidang{}, result.Error
	}
	return jadwal, nil
}

func (repo *JadwalSidangRepo) GetJadwalSidangByBatch(idBatch string) (output []entity.JadwalSidang, err error) {
	var jadwalSidang []entity.JadwalSidang
	result := repo.DBRead.Where("id_batch = ?", idBatch).Find(&jadwalSidang)
	if result.Error != nil {
		return nil, err
	}
	return jadwalSidang, nil
}

func (repo *JadwalSidangRepo) GetLatestJadwalSidang() (output []entity.JadwalSidang, err error) {
	var jadwalSidang []entity.JadwalSidang
	err = repo.DBRead.
		Where("id_batch = (SELECT MAX(time) FROM jadwal_sidangs)").
		Find(&jadwalSidang).Error
	if err != nil {
		return nil, err
	}
	return jadwalSidang, nil
}
func (repo *JadwalSidangRepo) GetJadwalSidangAll() (output []entity.JadwalSidang, err error) {
	var jadwalSidang []entity.JadwalSidang
	result := repo.DBRead.Find(&jadwalSidang)
	if result.Error != nil {
		return nil, err
	}
	return jadwalSidang, nil
}
func (repo *JadwalSidangRepo) UpdateJadwalSidang(jadwalSidang entity.JadwalSidang) (entity.JadwalSidang, error) {
	err := repo.DBWrite.Save(&jadwalSidang).Error
	if err != nil {
		return jadwalSidang, err
	}
	return jadwalSidang, nil
}

func (repo *JadwalSidangRepo) DeleteJadwalSidang(id string) (entity.JadwalSidang, error) {
	jadwalSidang := entity.JadwalSidang{ID: id}
	err := repo.DBWrite.Delete(&jadwalSidang).Error

	if err != nil {
		return jadwalSidang, err
	}

	return jadwalSidang, err
}
