package repository

import (
	"time"

	"github.com/google/uuid"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/entity"
	"gorm.io/gorm"
)

type PendaftaranRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewPendaftaranRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *PendaftaranRepo {
	return &PendaftaranRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *PendaftaranRepo) NewPendaftaran(pendaftaran entity.Pendaftaran) (output entity.Pendaftaran, err error) {
	pendaftaran.Id = uuid.New().String()
	pendaftaran.CreatedAt = time.Now()
	pendaftaran.UpdatedAt = time.Now()

	result := repo.DBWrite.Create(&pendaftaran)
	if result.Error != nil {
		return entity.Pendaftaran{}, result.Error
	}
	return pendaftaran, nil
}

func (repo *PendaftaranRepo) GetPendaftaran() (output []entity.Pendaftaran, err error) {
	var batches []entity.Pendaftaran
	result := repo.DBRead.Find(&batches)
	if result.Error != nil {
		return nil, err
	}
	return batches, nil
}

func (repo *PendaftaranRepo) DeletePendaftaran(id string) (entity.Pendaftaran, error) {
	batch := entity.Pendaftaran{Id: id}
	err := repo.DBWrite.Delete(&batch).Error

	if err != nil {
		return batch, err
	}

	return batch, err
}

func (repo *PendaftaranRepo) GetPendaftaranByIdDosen(id string) (output []entity.Pendaftaran, err error) {
	var pendaftaran []entity.Pendaftaran
	result := repo.DBRead.Where("id = ?", id).Find(&pendaftaran)
	if result.Error != nil {
		return nil, err
	}
	return pendaftaran, nil
}

func (repo *PendaftaranRepo) SetStatus(id string, val bool) (output []entity.Pendaftaran, err error) {
	var pendaftaran entity.Pendaftaran
	result := repo.DBRead.Where("id = ?", id).First(&pendaftaran)
	if result.Error != nil {
		return nil, result.Error
	}

	pendaftaran.Status = &val
	pendaftaran.UpdatedAt = time.Now()
	if val {
		now := time.Now()
		pendaftaran.DecidedAt = &now
	} else {
		pendaftaran.DecidedAt = nil
	}

	result = repo.DBRead.Save(&pendaftaran)
	if result.Error != nil {
		return nil, result.Error
	}

	return []entity.Pendaftaran{pendaftaran}, nil
}

func (repo *PendaftaranRepo) SetTimeInterview(id string, val time.Time) (output []entity.Pendaftaran, err error) {
	var pendaftaran entity.Pendaftaran
	result := repo.DBRead.Where("id = ?", id).First(&pendaftaran)
	if result.Error != nil {
		return nil, result.Error
	}

	pendaftaran.UpdatedAt = time.Now()
	pendaftaran.InterviewAt = &val

	result = repo.DBRead.Save(&pendaftaran)
	if result.Error != nil {
		return nil, result.Error
	}

	return []entity.Pendaftaran{pendaftaran}, nil
}
