package usecase

import (
	"time"

	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/entity"
)

type PendaftaranUsecase interface {
	AddPendaftaran(pendaftaran entity.Pendaftaran) (entity.Pendaftaran, error)
	GetPendaftaran() ([]entity.Pendaftaran, error)
	DeletePendaftaran(id string) (entity.Pendaftaran, error)
	GetPendaftaranByIdDosen(id string) ([]entity.Pendaftaran, error)
	UpdateStatusPendaftaran(id string, val bool) ([]entity.Pendaftaran, error)
	UpdateWaktuInterview(id string, val time.Time) ([]entity.Pendaftaran, error)
}

type PendaftaranUc struct {
	pendaftaranRepo PendaftaranRepository
}

func NewPendaftaranUseCase(pendaftaranRepo PendaftaranRepository) *PendaftaranUc {
	return &PendaftaranUc{
		pendaftaranRepo: pendaftaranRepo,
	}
}

func (uc *PendaftaranUc) AddPendaftaran(pendaftaran entity.Pendaftaran) (entity.Pendaftaran, error) {
	newPendaftaran, err := uc.pendaftaranRepo.NewPendaftaran(pendaftaran)

	if err != nil {
		return newPendaftaran, err
	}

	return newPendaftaran, nil

}

func (uc *PendaftaranUc) GetPendaftaran() ([]entity.Pendaftaran, error) {
	return uc.pendaftaranRepo.GetPendaftaran()
}

func (uc *PendaftaranUc) DeletePendaftaran(id string) (entity.Pendaftaran, error) {
	return uc.pendaftaranRepo.DeletePendaftaran(id)
}

func (uc *PendaftaranUc) GetPendaftaranByIdDosen(id string) ([]entity.Pendaftaran, error) {
	return uc.pendaftaranRepo.GetPendaftaranByIdDosen(id)
}

func (uc *PendaftaranUc) UpdateStatusPendaftaran(id string, val bool) ([]entity.Pendaftaran, error) {
	return uc.pendaftaranRepo.SetStatus(id, val)
}

func (uc *PendaftaranUc) UpdateWaktuInterview(id string, val time.Time) ([]entity.Pendaftaran, error) {
	return uc.pendaftaranRepo.SetTimeInterview(id, val)
}
