package repository

import (
	"errors"
	"time"

	"gitlab.informatika.org/k-01-11/graduit-be/src/models"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/status_mahasiswa/entity"
	"gorm.io/gorm"
)

type StatusMahasiswaRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewStatusMahasiswaRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *StatusMahasiswaRepo {
	return &StatusMahasiswaRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *StatusMahasiswaRepo) GetStatusMahasiswaByNIM(nim string) (output []entity.StatusMahasiswa, err error) {
	var statusMahasiswa entity.StatusMahasiswa
	var statusPendaftaran entity.StatusPendaftaran
	var statusBimbingan entity.StatusBimbingan
	var statusSeminar entity.StatusSeminar
	var statusSidang entity.StatusSidang

	var mahasiswa models.Mahasiswa
	var dosenPembimbing models.Dosen
	var topik models.Topik
	var judul models.UsulanJudul
	var pendaftaran models.Pendaftaran

	statusPendaftaran.Status = true

	// Get Mahasiswa
	result := repo.DBRead.Where("nim = ?", nim).First(&mahasiswa)
	if result.Error != nil {
		return nil, errors.New("mahasiswa tidak ditemukan")
	}
	statusMahasiswa.IdMahasiswa = mahasiswa.IdMahasiswa
	statusMahasiswa.NIM = mahasiswa.NIM
	statusMahasiswa.Nama = mahasiswa.Name

	// Get Pendaftaran

	// conditions 1 : already got supervisor
	result = repo.DBRead.Where("id_mahasiswa = ? AND status = TRUE", mahasiswa.IdMahasiswa).First(&pendaftaran).Order("updated_at DESC")
	if result.Error != nil {
		if errors.Is(result.Error, gorm.ErrRecordNotFound) {

			// conditions 2 : already apply but not accepted, yet
			result = repo.DBRead.Where("id_mahasiswa = ? AND (status = FALSE OR status IS NULL)", mahasiswa.IdMahasiswa).First(&pendaftaran).Order("updated_at DESC")
			if result.Error != nil {
				if errors.Is(result.Error, gorm.ErrRecordNotFound) {
					// conditions 3 : not applied yeet
					statusPendaftaran.Status = false
				}
			}
		}
	}

	if statusPendaftaran.Status {

		// Get Dosen
		result = repo.DBRead.Where("id_dosen = ?", mahasiswa.IdDosen).First(&dosenPembimbing)
		if result.Error != nil {
			return nil, errors.New("dosen tidak ditemukan")
		}

		// Get Topik
		result = repo.DBRead.Where("id = ?", pendaftaran.IdTopik).First(&topik)
		if result.Error != nil {
			return nil, errors.New("topik tidak ditemukan")
		}

		// Get Judul
		result = repo.DBRead.Where("id_pendaftaran = ?", pendaftaran.Id).First(&judul)
		if result.Error != nil {
			return nil, errors.New("judul tidak ditemukan")
		}

		statusPendaftaran.Topik = topik.Judul
		statusPendaftaran.Judul = judul.Judul
		statusPendaftaran.DosenPembimbing = dosenPembimbing.Name
		statusPendaftaran.PengirimanRegistrasi = pendaftaran.CreatedAt
		statusPendaftaran.PersetujuanDosenPembimbing = *pendaftaran.DecidedAt
		statusPendaftaran.JadwalInterview = *pendaftaran.InterviewAt
		statusPendaftaran.PengesahanDosenPembimbing = *pendaftaran.Status

	}

	statusMahasiswa.StatusPendaftaran = statusPendaftaran

	// below this still uncertain
	statusBimbingan.JumlahBimbingan = 10
	statusMahasiswa.StatusBimbingan = statusBimbingan

	statusSeminar.Status = true
	statusSeminar.DosenPenguji = "Dr. Ir. Rinaldi Munir, M.T."
	statusSeminar.JadwalSeminar = time.Now()
	statusSeminar.Ruangan = "7609"
	statusMahasiswa.StatusSeminar = statusSeminar

	statusSidang.Status = false
	statusMahasiswa.StatusSidang = statusSidang

	return []entity.StatusMahasiswa{statusMahasiswa}, nil
}
