package transport

import (
	"net/http"

	"github.com/labstack/echo"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/status_mahasiswa/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/status_mahasiswa/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/status_mahasiswa/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminStatusMahasiswaHandler struct {
	statusMahasiswaUsecase usecase.StatusMahasiswaUseCase
}

func NewAdminStatusMahasiswaHandler(cfg config.StatusMahasiswaTransportConfig) *AdminStatusMahasiswaHandler {
	statusMahasiswaRepository := repository.NewStatusMahasiswaRepository(cfg.DBWrite, cfg.DBRead)
	statusMahasiswaUsecase := usecase.NewStatusMahasiswaUsecase(statusMahasiswaRepository)

	return &AdminStatusMahasiswaHandler{
		statusMahasiswaUsecase: statusMahasiswaUsecase,
	}

}

func (t *AdminStatusMahasiswaHandler) MountAdmin(group *echo.Group) {
	group.GET("/status-mahasiswa", t.GetStatusMahasiswa)
}

func (t *AdminStatusMahasiswaHandler) GetStatusMahasiswa(c echo.Context) error {
	nim := c.QueryParam("nim")
	if nim == "" {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "insert nim", nil))
	}

	statusMahasiswa, err := t.statusMahasiswaUsecase.GetStatusMahasiswa(nim)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Status mahasiswa successfully retreived", statusMahasiswa[0]))
}
