package usecase

import (
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/topic_allocation/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type TopicUsecase interface {
	AddTopic(topic entity.Topic) (entity.Topic, error)
	GetTopic(param utils.PageLimitSearchID) ([]entity.Topic, error)
	GetTopicByID(param string) (entity.Topic, error)
	UpdateTopic(topic entity.Topic) (entity.Topic, error)
	DeleteTopic(param string) (error)
}

type TopicUc struct {
	topicAllocationRepo TopicAllocationRepo
}

func NewTopicUc(topicAllocationRepo TopicAllocationRepo) *TopicUc {
	return &TopicUc{
		topicAllocationRepo: topicAllocationRepo,
	}
}

func (uc *TopicUc) AddTopic(topic entity.Topic) (entity.Topic, error) {
	newTopic, err := uc.topicAllocationRepo.NewTopic(topic)

	if err != nil {
		return entity.Topic{}, err
	}

	return newTopic, nil
}

func (uc *TopicUc) GetTopic(param utils.PageLimitSearchID) ([]entity.Topic, error) {
	return uc.topicAllocationRepo.GetTopic(param.Page, param.Limit, param.Search, param.ID)
}

func (uc *TopicUc) GetTopicByID(param string) (entity.Topic, error) {
	return uc.topicAllocationRepo.GetTopicByID(param)
}

func (uc *TopicUc) UpdateTopic(topic entity.Topic) (entity.Topic, error) {
	updatedTopic, err := uc.topicAllocationRepo.UpdateTopic(topic)

	if err != nil {
		return entity.Topic{}, err
	}

	return updatedTopic, nil
}

func (uc *TopicUc) DeleteTopic(param string) (error) {
	err := uc.topicAllocationRepo.DeleteTopic(param)

	if err != nil {
		return err
	}

	return nil
}